<?php

namespace SM\Service;

use SM\Factory\SmAgrupamentoEscopoFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;
use SM\Service\FindService;

class SmAgrupamentoEscopoService
{

    protected $AgrupamentoFactory;
    protected $Environment;
    protected $newCode;
    protected $connection;
    protected $FindService;

    public function __construct(EntityManager $entityManager, SmAgrupamentoEscopoFactory $AgrupamentoFactory, Environment $Environment, NewCode $newCode, FindService $FindService)
    {
        $this->connection         = $entityManager->getConnection();
        $this->AgrupamentoFactory = $AgrupamentoFactory;
        $this->Environment        = $Environment;
        $this->newCode            = $newCode;
        $this->FindService        = $FindService;
    }


    public function getAgrupamentoEscopoFaturamento()
    {
        return $this->AgrupamentoFactory->getAgrupamentoEscopoFaturamento();
    }

    public function insertAgrupamentoTipoOrca($parametros)
    {
        $this->connection->beginTransaction();
        try {
            $cdOperador = $this->Environment->getCdOperador();
            $NRORG = $this->Environment->getNrOrg();

            if (!isset($parametros['NRAGRUPAMENTOTIPOORCA'])) {
                $this->AgrupamentoFactory->insertAgrupamentoTipoOrca(array(
                    "NRAGRUPAMENTOTIPOORCA"     => $this->newCode->getCodeByTableName("FCT_AGRUPAMENTOTIPOORCA", $NRORG, 1, 10, true),
                    "NMAGRUPAMENTOTIPOORCA"     => $parametros['NMAGRUPAMENTOTIPOORCA'],
                    "DSOBSAGRUPTIPOORCA"        => isset($parametros['DSOBSAGRUPTIPOORCA']) ? $parametros['DSOBSAGRUPTIPOORCA'] : null,
                    "CDOPERINCLUSAO"            => $cdOperador,
                    "NRORGINCLUSAO"             => $NRORG,
                    "NRORG"                     => $NRORG,
                    "IDTPFATURA"                => $parametros['IDTPFATURA'],
                    "CDFILIAL"                  => $parametros['CDFILIAL'],
                    "CDEMPRESA"                 => $parametros['CDEMPRESA'],
                    "CDTIPOOPERFATURA"          => $parametros['CDTIPOOPER'],
                    "IDNATUOPER"                => isset($parametros['IDNATUOPER']) ? $parametros['IDNATUOPER'] : null,
                    "CDSERIE"                   => $parametros['CDSERIE'],
                    "CDINSCMUNI"                => $parametros['CDINSCMUNI'],
                    "CDINSCESTA"                => $parametros['CDINSCESTA'],
                    "IDMUNIPRESTSERV"           => isset($parametros['IDMUNIPRESTSERV']) ? $parametros['IDMUNIPRESTSERV'] : null,
                    "IDTIPONATUOPER"            => isset($parametros['IDTIPONATUOPER']) ? $parametros['IDTIPONATUOPER'] : null,
                ));
            } else {
                $this->AgrupamentoFactory->updateAgrupamentoTipoOrca(array(
                    "NRAGRUPAMENTOTIPOORCA"     => $parametros['NRAGRUPAMENTOTIPOORCA'],
                    "NMAGRUPAMENTOTIPOORCA"     => $parametros['NMAGRUPAMENTOTIPOORCA'],
                    "DSOBSAGRUPTIPOORCA"        => isset($parametros['DSOBSAGRUPTIPOORCA']) ? $parametros['DSOBSAGRUPTIPOORCA'] : null,
                    "CDOPERULTATU"              => $cdOperador,
                    "NRORGULTATU"               => $NRORG,
                    "IDTPFATURA"                => $parametros['IDTPFATURA'],
                    "CDFILIAL"                  => $parametros['CDFILIAL'],
                    "CDEMPRESA"                 => $parametros['CDEMPRESA'],
                    "CDTIPOOPERFATURA"          => $parametros['CDTIPOOPER'],
                    "IDNATUOPER"                => isset($parametros['IDNATUOPER']) ? $parametros['IDNATUOPER'] : null,
                    "CDSERIE"                   => $parametros['CDSERIE'],
                    "CDINSCMUNI"                => $parametros['CDINSCMUNI'],
                    "CDINSCESTA"                => $parametros['CDINSCESTA'],
                    "IDMUNIPRESTSERV"           => isset($parametros['IDMUNIPRESTSERV']) ? $parametros['IDMUNIPRESTSERV'] : null,
                    "IDTIPONATUOPER"            => isset($parametros['IDTIPONATUOPER']) ? $parametros['IDTIPONATUOPER'] : null,
                ));
            }
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function getSerieNotaFiscal($parametros)
    {
        return $this->AgrupamentoFactory->getSerieNotaFiscal(array(
            "CDEMPRESA" => isset($parametros['CDEMPRESA']) ? $parametros['CDEMPRESA'] : null,
        ));
    }

    public function getMunicipio($parametros)
    {
        $NRORG = $this->Environment->getNrOrg();

        return $this->AgrupamentoFactory->getMunicipio(array(
            "CDFILIAL"      => isset($parametros['CDFILIAL']) ? $parametros['CDFILIAL'] : null,
            "NRORG"         => $NRORG,
            "NRPARCNEGOCIO" => null,
            "NRESTRUTURAM"  => null,
        ));
    }

    public function getEscoposParaAgrupar($parametros)
    {
        return $this->AgrupamentoFactory->getEscoposParaAgrupar(array(
            "IDTPFATURA"                => $parametros['IDTPFATURA'],
            "CDFILIAL"                  => $parametros['CDFILIAL'],
            "CDTIPOOPER"                => $parametros['CDTIPOOPER'],
            "IDNATUOPER"                => isset($parametros['IDNATUOPER']) ? $parametros['IDNATUOPER'] : null,
            "CDSERIE"                   => $parametros['CDSERIE'],
            "CDINSCMUNI"                => $parametros['CDINSCMUNI'],
            "CDINSCESTA"                => $parametros['CDINSCESTA'],
            "IDMUNIPRESTSERV"           => isset($parametros['IDMUNIPRESTSERV']) ? $parametros['IDMUNIPRESTSERV'] : null,
            "IDTIPONATUOPER"            => isset($parametros['IDTIPONATUOPER']) ? $parametros['IDTIPONATUOPER'] : null,
        ));
    }

    public function getEscoposAgrupados($parametros)
    {
        return $this->AgrupamentoFactory->getEscoposAgrupados(array(
            "NRAGRUPAMENTOTIPOORCA" => $parametros['NRAGRUPAMENTOTIPOORCA'],
        ));
    }

    public function agruparEscopo($parametros)
    {
        $this->connection->beginTransaction();
        try {
            $cdOperador = $this->Environment->getCdOperador();
            $NRORG = $this->Environment->getNrOrg();

            foreach ($parametros['NRTIPOORCAMENTO'] as $tipoorca) {
                $this->AgrupamentoFactory->agruparEscopo(array(
                    "NRRELACAGRUPTIPOORCA"  => $this->newCode->getCodeByTableName("FCT_RELACAGRUPTIPOORCA", $NRORG, 1, 10, true),
                    "NRAGRUPAMENTOTIPOORCA" => $parametros['NRAGRUPAMENTOTIPOORCA'],
                    "NRTIPOORCAMENTO"       => $tipoorca,
                    "NRORG"                 => $NRORG,
                    "CDOPERINCLUSAO"        => $cdOperador,
                    "NRORGINCLUSAO"         => $NRORG,
                    
                ));
            }

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function excluirEscoposAgrupados($parametros)
    {
        $this->connection->beginTransaction();
        try {
            $this->AgrupamentoFactory->excluirEscoposAgrupados($this->FindService->formatarArray($parametros['NRRELACAGRUPTIPOORCA']));
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }
}
