import { Utilities } from '@/util/Utilities';
import {
    Form, Modal, ModalService, Select, Date, LoadingService, TextInput, DialogService, DateRange,
} from '@zeedhi/common';
import { Http, Metadata } from '@zeedhi/core';
import { TekGrid } from '@zeedhi/teknisa-components-common';
import moment from 'moment';

export class AgendaServicosController {
    private LogModal!: Modal;

    private FilterModal!: Modal;

    private ZerarModal!: Modal;

    private EditarModal!: Modal;

    public campoModal: any = {};

    public currentRow: any = {};

    public mascaraSaldo: String = '';

    public eventFilter({ event }: any) {
        event.preventDefault();
        this.abrirModalFilterClick();
    }

    public async abrirModalFilterClick() {
        await this.exibirModalFilter();
        Metadata.getInstance<Select>('formFilterAgendaServicos-CDCLIENTE').value = this.campoModal.cdcliente;
        Metadata.getInstance<Select>('formFilterAgendaServicos-NRPROPOSFAS').value = this.campoModal.nrproposfas;
        Metadata.getInstance<Select>('contratos-sm').value = this.campoModal.contrato;
        Metadata.getInstance<DateRange>('periodo-data-agendamento').value = this.campoModal.dataAgendamento;
    }

    public async abrirModalFilter() {
        await this.exibirModalFilter();
    }

    public async exibirModalFilter() {
        this.FilterModal = await ModalService.createFromJson('agendaServicos/modal/filter_modal_agenda_servicos', true);
        this.FilterModal.show();
    }

    public fecharModalFilter() {
        this.FilterModal.hide();
        this.FilterModal?.destroy();
    }

    public setFilterCampoProposta({ component }: any) {
        const proposta = Metadata.getInstance<Select>('formFilterAgendaServicos-NRPROPOSFAS');

        if (component.value) {
            proposta.datasource.setFilter({
                CDCLIENTE: component.value,
            });
            proposta.disabled = false;
        } else {
            proposta.disabled = true;
            proposta.value = null;
        }
    }

    public filterGrid() {
        const cdcliente = Metadata.getInstance<Select>('formFilterAgendaServicos-CDCLIENTE').value;
        const nrproposfas = Metadata.getInstance<Select>('formFilterAgendaServicos-NRPROPOSFAS');
        const grid = Metadata.getInstance<TekGrid>('tek-grid-agenda-servicos');
        const contrato = Metadata.getInstance<Select>('contratos-sm').value;
        const dataAgendamento = Metadata.getInstance<DateRange>('periodo-data-agendamento').value;

        this.campoModal = {
            cdcliente,
            nrproposfas,
            contrato,
            dataAgendamento,
        };

        grid.datasource.setFilter({
            CDCLIENTE: cdcliente,
            NRPROPOSFAS: nrproposfas.value,
            NRVERPROPFAS: (nrproposfas.value === null) ? null : nrproposfas.selectValue.NRVERPROPFAS,
            NRSEQCONTRATO: contrato,
            USR_DTAGENDZERASALDO: dataAgendamento,
        });
        this.fecharModalFilter();
    }

    public async abrirModalZerar({ row }: any) {
        await this.exibirModalZerar();
        Metadata.getInstance<Date>('formZerarAgendaServicos-USR_DTAGENDZERASALDO').value = moment().format('DD/MM/YYYY');
        if (row !== undefined) {
            this.currentRow = {
                USR_QTSERVCONT_BKP: row.USR_QTSERVCONT_BKP,
                USR_SALDO_BKP: row.USR_SALDO_BKP,
                CDTIPOSERV: row.CDTIPOSERV,
                CDCLIENTE: row.CDCLIENTE,
                NRPROPOSFAS: row.NRPROPOSFAS,
                NRVERPROPFAS: row.NRVERPROPFAS,
                SALDO: row.SALDO,
                QTSERVPEDC: row.QTSERVPEDC,
                NRSERVCONT: row.NRSERVCONT,
                BOTTONGRID: true,
                QTHRSERVFATU: row.QTHRSERVFATU,
                ROWNUM: row.ROWNUM,
            };
        }
    }

    public async exibirModalZerar() {
        this.ZerarModal = await ModalService.createFromJson('agendaServicos/modal/zerar_modal_agenda_servicos', true);
        this.ZerarModal.show();
    }

    public fecharModalZerar() {
        this.ZerarModal.hide();
        this.ZerarModal?.destroy();
    }

    public zerarSaldoDeHoras() {
        if (this.currentRow.BOTTONGRID === true) {
            const form = Metadata.getInstance<Form>('formZerarAgendaServicos');
            if (Utilities.validateForm(form)) {
                const grid = Metadata.getInstance<TekGrid>('tek-grid-agenda-servicos');

                const saldo = this.convertMinutes(this.currentRow.SALDO);
                const contratadas = this.convertMinutes(this.currentRow.QTSERVPEDC);
                const servFatu = this.convertMinutes(this.currentRow.QTHRSERVFATU);

                let dataInput = Metadata.getInstance<Date>('formZerarAgendaServicos-USR_DTAGENDZERASALDO').value.split(' ')[0];
                dataInput = this.formataStringData(dataInput);
                const dsoperacao = `SALDO DE HORAS ALTERADO - Anterior: ${this.currentRow.SALDO} / Novo: 00:00`;
                const row = {
                    USR_QTSERVCONT_BKP: this.currentRow.USR_QTSERVCONT_BKP,
                    USR_SALDO_BKP: this.currentRow.USR_SALDO_BKP,
                    QTSERVPEDC: contratadas,
                    QTHRSERVFATU: servFatu,
                    SALDO: saldo,
                    CDTIPOSERV: this.currentRow.CDTIPOSERV,
                    CDCLIENTE: this.currentRow.CDCLIENTE,
                    NRPROPOSFAS: this.currentRow.NRPROPOSFAS,
                    NRVERPROPFAS: this.currentRow.NRVERPROPFAS,
                    NRSERVCONT: this.currentRow.NRSERVCONT,
                    DSOPERACAO: dsoperacao,
                    ROWNUM: this.currentRow.ROWNUM,
                };
                LoadingService.show();
                Http.post('/zerarSaldoDeHoras', {
                    row,
                    USR_DTAGENDZERASALDO: dataInput,
                }).then(() => {
                    Utilities.showSuccessMessage();
                    grid.datasource.get();
                    this.fecharModalZerar();
                }).catch((error) => {
                    if (error.response.data.error === 'ERRO') {
                        Utilities.showDialogMessage('error', 'Operação bloqueada!', 'A data não pode ser inferior à data atual');
                    } else {
                        Utilities.showErrorMessage(error);
                    }
                }).finally(() => {
                    LoadingService.hide();
                });
            }
        } else {
            this.zerarSaldoDeHorasAction();
        }
    }

    public zerarSaldoDeHorasAction() {
        const form = Metadata.getInstance<Form>('formZerarAgendaServicos');
        let dataInput = Metadata.getInstance<Date>('formZerarAgendaServicos-USR_DTAGENDZERASALDO').value.split(' ')[0];
        dataInput = this.formataStringData(dataInput);
        if (Utilities.validateForm(form)) {
            const grid = Metadata.getInstance<TekGrid>('tek-grid-agenda-servicos');
            LoadingService.show();
            Http.post('/zerarSaldoDeHorasAction', {
                USR_DTAGENDZERASALDO: dataInput,
                row: grid.selectedRows,
            }).then(() => {
                Utilities.showSuccessMessage();
                grid.datasource.get();
                grid.selectedRows = [];
                this.fecharModalZerar();
            }).catch((error) => {
                if (error.response.data.error === 'ERRO') {
                    Utilities.showDialogMessage('error', 'Operação bloqueada!', 'A data não pode ser inferior à data atual');
                } else {
                    Utilities.showErrorMessage(error);
                }
            }).finally(() => {
                LoadingService.hide();
            });
        }
    }

    public convertMinutes(hora: any) {
        const f0 = hora.split(':');
        const minutes = ((Number(f0[0]) * 60) + Number(f0[1]));
        const horas = minutes / 60;
        return horas;
    }

    public formataStringData(data: any) {
        const dia = data.split('/')[0];
        const mes = data.split('/')[1];
        const ano = data.split('/')[2];
        return `${(`0${dia}`).slice(-2)}-${(`0${mes}`).slice(-2)}-${ano}`;
    }

    public async abrirModalEditar({ row }: any) {
        await this.exibirModalEditar();
        const [contratada, saldo] = this.padronizarHora(row.QTSERVPEDC, row.SALDO);
        Metadata.getInstance<TextInput>('formEditarAgendaServicos-HRCONTRATADA').value = contratada;
        Metadata.getInstance<TextInput>('formEditarAgendaServicos-NOVO_SALDO').value = saldo;
        this.currentRow = {
            USR_QTSERVCONT_BKP: row.USR_QTSERVCONT_BKP,
            USR_SALDO_BKP: row.USR_SALDO_BKP,
            QTSERVCONT_BKP: row.QTSERVCONT,
            QTHRAPROFATU: row.QTHRAPROFATU,
            CDTIPOSERV: row.CDTIPOSERV,
            CDCLIENTE: row.CDCLIENTE,
            NRPROPOSFAS: row.NRPROPOSFAS,
            NRVERPROPFAS: row.NRVERPROPFAS,
            QTHRSERVPREST: row.QTHRSERVPREST,
            SALDO: row.SALDO,
            QTSERVPEDC: row.QTSERVPEDC,
            QTHRSERVFATU: row.QTHRSERVFATU,
            NRSERVCONT: row.NRSERVCONT,
            ROWNUM: row.ROWNUM,
        };
    }

    public padronizarHora(hrContratada: String, novoSaldo: String) {
        // this.mascaraSaldo = 'NNNN:NN';
        let hrContratadaPadronizada = hrContratada;
        let saldoPadronizado = novoSaldo;

        if (hrContratadaPadronizada.includes(':')) {
            const [horacontrada, minuto] = hrContratadaPadronizada.split(':');
            hrContratadaPadronizada = `${horacontrada.toString().padStart(4, '0')}:${minuto.padStart(2, '0')}`;
        } else if (hrContratadaPadronizada.length === 5 || hrContratadaPadronizada.length === 6) {
            hrContratadaPadronizada = `${hrContratadaPadronizada.substring(0, 4)}:${hrContratadaPadronizada.substring(4, 6)}`;
        }

        if (saldoPadronizado.includes(':')) {
            const [horacontrada, minuto] = saldoPadronizado.split(':');
            if (saldoPadronizado.includes('-')) {
                this.mascaraSaldo = '-NNNN:NN';
                saldoPadronizado = `-${horacontrada.toString().substring(1).padStart(4, '0')}:${minuto.padStart(2, '0')}`;
            } else {
                this.mascaraSaldo = 'NNNN:NN';
                saldoPadronizado = `${horacontrada.toString().padStart(4, '0')}:${minuto.padStart(2, '0')}`;
            }
        } else if (saldoPadronizado.length === 5 || saldoPadronizado.length === 6) {
            saldoPadronizado = `${saldoPadronizado.substring(0, 4)}:${saldoPadronizado.substring(4, 6)}`;
        }

        return [hrContratadaPadronizada, saldoPadronizado];
    }

    public async exibirModalEditar() {
        this.EditarModal = await ModalService.createFromJson('agendaServicos/modal/editar_modal_agenda_servicos', true);
        this.EditarModal.show();
    }

    public fecharModalEditar() {
        this.EditarModal.hide();
        this.EditarModal?.destroy();
    }

    public editarSaldoDeHoras() {
        const form = Metadata.getInstance<Form>('formEditarAgendaServicos');
        if (Utilities.validateForm(form)) {
            const grid = Metadata.getInstance<TekGrid>('tek-grid-agenda-servicos');
            const nvHrsContratadas = Metadata.getInstance<TextInput>('formEditarAgendaServicos-HRCONTRATADA').value;
            const nvSaldoHoras = Metadata.getInstance<TextInput>('formEditarAgendaServicos-NOVO_SALDO').value;

            const horaPadronizada = this.padronizarHora(nvHrsContratadas, nvSaldoHoras);

            const horasCotratadas = horaPadronizada[0];
            const saldohoras = horaPadronizada[1];

            const dsoperacao = `SALDO DE HORAS ALTERADO - Anterior: ${this.currentRow.SALDO} / Novo: ${saldohoras}`;
            const contratadas = this.convertMinutes(horasCotratadas);

            const row = {
                USR_QTSERVCONT_BKP: this.currentRow.USR_QTSERVCONT_BKP,
                USR_SALDO_BKP: this.currentRow.USR_SALDO_BKP,
                QTSERVCONT_BKP: this.currentRow.QTSERVCONT_BKP,
                QTSERVCONT: this.currentRow.QTSERVCONT,
                QTHRAPROFATU: this.currentRow.QTHRAPROFATU,
                SALDO_EDITADO: nvSaldoHoras,
                QTSERVPEDC: contratadas,
                CDTIPOSERV: this.currentRow.CDTIPOSERV,
                CDCLIENTE: this.currentRow.CDCLIENTE,
                NRPROPOSFAS: this.currentRow.NRPROPOSFAS,
                NRVERPROPFAS: this.currentRow.NRVERPROPFAS,
                DSOPERACAO: dsoperacao,
                NRSERVCONT: this.currentRow.NRSERVCONT,
                USR_DTAGENDZERASALDO: null,
                ROWNUM: this.currentRow.ROWNUM,
            };
            LoadingService.show();
            Http.post('/editarSaldoDeHoras', {
                row,
            }).then(() => {
                Utilities.showSuccessMessage();
                grid.datasource.get();
                this.fecharModalEditar();
            }).catch((error) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                LoadingService.hide();
            });
        }
    }

    public showConfirmDialogReverterSaldo({ row }: any) {
        DialogService.show({
            name: 'warning_dialog',
            type: 'warning',
            title: 'Confirmação',
            text: 'Deseja realmente voltar a situação contratual inicial do saldo de horas?',
            buttons: [
                {
                    name: 'button-no',
                    component: 'ZdButton',
                    label: 'Não',
                    events: {
                        click: () => DialogService.hide(),
                    },
                    outline: true,
                },
                {
                    name: 'button-yes',
                    component: 'ZdButton',
                    label: 'Sim',
                    events: {
                        click: () => {
                            LoadingService.show();
                            if (row) {
                                this.reverterSaldoHorasBotton(row);
                            } else {
                                this.reverterSaldoHorasAction();
                            }
                            DialogService.hide();
                        },
                    },
                },
            ],
        });
    }

    public converterSegundosFormato(segundos: any) {
        const h = segundos / 3600;
        const resto = segundos % 3600;
        const m = resto / 60;
        return `${h}:${m}`;
    }

    public reverterSaldoHorasBotton(rowGrid: any) {
        const grid = Metadata.getInstance<TekGrid>('tek-grid-agenda-servicos');
        const saldo = this.converterSegundosFormato(rowGrid.USR_QTSERVCONT_BKP * 3600);
        const dsoperacao = `SALDO DE HORAS ALTERADO - Anterior: ${rowGrid.SALDO} / Novo: ${saldo}`;
        const row = {
            USR_QTSERVCONT_BKP: rowGrid.USR_QTSERVCONT_BKP,
            USR_SALDO_BKP: rowGrid.USR_SALDO_BKP,
            QTSERVCONT: rowGrid.QTSERVCONT,
            QTHRAPROFATU: rowGrid.QTHRAPROFATU,
            CDTIPOSERV: rowGrid.CDTIPOSERV,
            CDCLIENTE: rowGrid.CDCLIENTE,
            NRPROPOSFAS: rowGrid.NRPROPOSFAS,
            NRVERPROPFAS: rowGrid.NRVERPROPFAS,
            NRSERVCONT: rowGrid.NRSERVCONT,
            DSOPERACAO: dsoperacao,
            ROWNUM: rowGrid.ROWNUM,
        };
        LoadingService.show();
        Http.post('/reverterSaldoHoras', {
            row,
        }).then(() => {
            Utilities.showSuccessMessage();
            grid.datasource.get();
        }).catch((error) => {
            if (error.response.data.error === 'ERRO') {
                Utilities.showDialogMessage('error', 'Operação bloqueada!', 'Saldo já esta com valor original contratado.');
            } else {
                Utilities.showErrorMessage(error);
            }
        }).finally(() => {
            LoadingService.hide();
        });
    }

    public reverterSaldoHorasAction() {
        const grid = Metadata.getInstance<TekGrid>('tek-grid-agenda-servicos');
        LoadingService.show();
        Http.post('/reverterSaldoHorasAction', {
            row: grid.selectedRows,
        }).then(() => {
            Utilities.showSuccessMessage();
            grid.datasource.get();
            grid.selectedRows = [];
        }).catch((error) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
        });
    }

    public async abrirModalLog({ row }: any) {
        await this.exibirModalLog();
        const gridLog = Metadata.getInstance<TekGrid>('tek-grid-modal-log');
        gridLog.datasource.setFilter({
            CDPESQITEM: `${row.NRPROPOSFAS}${row.NRVERPROPFAS}${row.CDTIPOSERV.split('_')[1]}${row.NRSERVCONT}`,
        });
    }

    public async exibirModalLog() {
        this.LogModal = await ModalService.createFromJson('agendaServicos/modal/log_modal_agenda_servicos', true);
        this.LogModal.show();
    }

    public fecharModalLog() {
        this.LogModal.hide();
        this.LogModal?.destroy();
    }

    public mudarHrContratada() {
        const nvHrsContratadas = Metadata.getInstance<TextInput>('formEditarAgendaServicos-HRCONTRATADA');
        // const saldoHoras = Metadata.getInstance<TextInput>('formEditarAgendaServicos-NOVO_SALDO');
        const saldoHoras: any = document.getElementById('formEditarAgendaServicos-NOVO_SALDO');
        const horaPadronizada = this.padronizarHora(this.currentRow.QTHRSERVFATU, saldoHoras.value);
        const hrSerPrestado = horaPadronizada[0];
        const nvSaldoHoras = horaPadronizada[1];
        if (nvSaldoHoras !== null && (nvSaldoHoras.length === 7 || nvSaldoHoras.length === 8)) {
            const segundos = this.converterHorasParaSegundos(nvSaldoHoras) + (this.converterHorasParaSegundos(hrSerPrestado));
            const retornoHora = this.converterSegundosFormato(segundos);
            const retorno = retornoHora.split(':');
            const hora = retorno[0];
            let minuto = retorno[1];
            minuto = `00${minuto}`.substr(-2, 2);
            nvHrsContratadas.value = `${Math.trunc(Number(hora)).toString().padStart(4, '0')}:${minuto}`;
        }
    }

    public converterHorasParaSegundos(horas: any) {
        const [hora, minuto] = horas.split(':');
        const segundos = ((Number(hora) * 60) + Number(minuto)) * 60;
        return segundos;
    }

    public mudarSaldo() {
        const nvSaldoHoras = Metadata.getInstance<TextInput>('formEditarAgendaServicos-NOVO_SALDO');
        const hrContratada = Metadata.getInstance<TextInput>('formEditarAgendaServicos-HRCONTRATADA').value;
        const horaPadronizada = this.padronizarHora(hrContratada, this.currentRow.QTHRSERVFATU);
        const nvHrsContratadas = horaPadronizada[0];
        const hrSerPrestado = horaPadronizada[1];

        if (nvHrsContratadas !== null && nvHrsContratadas.length === 7) {
            const segundos = this.converterHorasParaSegundos(nvHrsContratadas) - (this.converterHorasParaSegundos(hrSerPrestado));
            if (segundos < 0) this.mascaraSaldo = '-NNNN:NN';
            const retornoHora = this.converterSegundosFormato(segundos);
            const retorno = retornoHora.split(':');
            const hora = retorno[0];
            let minuto = (retorno[1].includes('-')) ? retorno[1].substring(1) : retorno[1];
            minuto = `00${minuto}`.substr(-2, 2);
            nvSaldoHoras.value = (retorno[0].includes('-')) ? `-${Math.trunc(Number(hora)).toString().substring(1).padStart(4, '0')}:${minuto}` : `${Math.trunc(Number(hora)).toString().padStart(4, '0')}:${minuto}`;
        }
    }
}
