import {
    Select, Tabs, TextInput, Form, Row,
} from '@zeedhi/common';
import { Utilities } from '@/util/Utilities';
import { Metadata } from '@zeedhi/core';

export class ServicoEscopoTabParametrosFaturamentoController {
    public expandir({ component }: any) {
        const form = Metadata.getInstance<Form>(component.name.split('_')[1]);
        form.isVisible = !form.isVisible;

        component.iconName = (component.iconName === 'mdi-chevron-down') ? 'mdi-chevron-right' : 'mdi-chevron-down';
    }

    public alteraVisibilidadeFormParametrosFaturamento() {
        const tipoFaturamento = Metadata.getInstance<Select>('parametroFaturamentoTab-IDTPFATURA').value;
        const tipoMedidaVariavel = Metadata.getInstance<Select>('parametroFaturamentoTab-CDTPMEDIDAVARIAVEL');
        const rowDadosVencimento = Metadata.getInstance<Row>('rowDadosVencimento');
        const formDadosVencimento = Metadata.getInstance<Form>('parametroFaturamentoTab-formEditDadosVencimento');
        const rowDadosParcelamento = Metadata.getInstance<Row>('rowEditDadosParcelamento');
        const formDadosParcelamento = Metadata.getInstance<Form>('parametroFaturamentoTab-formEditDadosParcelamento');
        const qtdMaxParcelas = Metadata.getInstance<TextInput>('parametroFaturamentoTab-QTPARCELASFATU');
        const qtdDiasApuracao = Metadata.getInstance<TextInput>('parametroFaturamentoTab-QTDIASAPURACAO');
        const tpCalcVencimento = Metadata.getInstance<Select>('parametroFaturamentoTab-IDTPCALCVENC');
        const diaVencimento = Metadata.getInstance<TextInput>('parametroFaturamentoTab-NRDIACALCVENC');
        const mesCalculoVencimento = Metadata.getInstance<Select>('parametroFaturamentoTab-IDMESCALCVENC');
        const qtdParcelas = Metadata.getInstance<TextInput>('parametroFaturamentoTab-QTPARCELASPARAUTO');
        const qtdIntervaloParcelas = Metadata.getInstance<TextInput>('parametroFaturamentoTab-QTDIASINTERVALOPARAUTO');
        const diaVencimentoParcela = Metadata.getInstance<TextInput>('parametroFaturamentoTab-NRDIAVENCIMENTOPARAUTO');
        const tempoCarencia = Metadata.getInstance<TextInput>('parametroFaturamentoTab-QTCARENCIADIASPARAUTO');
        const tabCompetencia = Metadata.getInstance<Tabs>('tabsInformationServicoEscopo').tabs[2];

        if (tipoFaturamento === 'X') {
            tipoMedidaVariavel.readonly = false;
        } else {
            tipoMedidaVariavel.readonly = true;
            tipoMedidaVariavel.value = null;
        }

        if (tipoFaturamento === 'P') {
            rowDadosVencimento.isVisible = true;
            formDadosVencimento.isVisible = true;
            rowDadosParcelamento.isVisible = true;
            formDadosParcelamento.isVisible = true;
            qtdMaxParcelas.addValidation('required', {});
            qtdDiasApuracao.addValidation('required', {});
            tpCalcVencimento.addValidation('required', {});
            mesCalculoVencimento.addValidation('required', {});
            diaVencimento.addValidation('required', {});
            qtdParcelas.readonly = false;
            qtdIntervaloParcelas.readonly = false;
            diaVencimentoParcela.readonly = false;
            tempoCarencia.readonly = false;
            tabCompetencia.isVisible = false;
        } else {
            rowDadosVencimento.isVisible = false;
            formDadosVencimento.isVisible = false;
            rowDadosParcelamento.isVisible = false;
            formDadosParcelamento.isVisible = false;
            qtdMaxParcelas.removeValidation('required');
            qtdDiasApuracao.removeValidation('required');
            tpCalcVencimento.removeValidation('required');
            mesCalculoVencimento.removeValidation('required');
            diaVencimento.removeValidation('required');
            qtdParcelas.readonly = true;
            qtdIntervaloParcelas.readonly = true;
            diaVencimentoParcela.readonly = true;
            tempoCarencia.readonly = true;
            qtdParcelas.value = null;
            qtdIntervaloParcelas.value = null;
            diaVencimentoParcela.value = null;
            tempoCarencia.value = null;
            tabCompetencia.isVisible = true;
        }
    }

    public atualizaNotaFiscal({ component }: any) {
        const cdSerieNf = Metadata.getInstance<Select>('parametroFaturamentoTab-CDSERIE');
        
        if (component.value) {
            cdSerieNf.datasource.setFilter({
                CDEMPRESA: component.datasource.currentRowValue.CDEMPRESA,
            });
        } else {
            cdSerieNf.datasource.setFilter({
                CDEMPRESA: null,
            });
        }

        Utilities.alertMessage('Atenção: Campo "Série da Nota Fiscal" foi recarregado', 'warning');
    }

    public controlarTipoCalculoVencimentoDadosGerais({ component }: any) {
        const idtpcalcvenc = component.value;
        const diaVencimento = Metadata.getInstance<TextInput>('parametroFaturamentoTab-NRDIACALCVENC');
        const mesCalculoVencimento = Metadata.getInstance<Select>('parametroFaturamentoTab-IDMESCALCVENC');
        const diasMinimosFatVenc = Metadata.getInstance<Select>('parametroFaturamentoTab-QTDIASFATUVENC');

        if (idtpcalcvenc === '1') { //  Dia fixo para a data de vencimento
            diaVencimento.label = 'Dia de Vencimento';
            
            mesCalculoVencimento.readonly = false;
            mesCalculoVencimento.addValidation('required', {});
            
            diasMinimosFatVenc.readonly = false;
            diasMinimosFatVenc.addValidation('required', {});
        } else if (idtpcalcvenc === '2') { // Quantidade de dias após a data de emissão
            diaVencimento.label = 'Quantidade de Dias';

            mesCalculoVencimento.readonly = true;
            mesCalculoVencimento.removeValidation('required');
            mesCalculoVencimento.value = '';
            
            diasMinimosFatVenc.readonly = true;
            diasMinimosFatVenc.removeValidation('required');
            mesCalculoVencimento.value = '';
        }
    }

    public alterarVisibilidadefieldNaturezaOperacao({ component } : any) {
        const idnatuoper = Metadata.getInstance<Select>('parametroFaturamentoTab-IDNATUOPER');

        if (component.value === 'F') {
            idnatuoper.readonly = false;
            idnatuoper.addValidation('required', {});
        } else {
            idnatuoper.readonly = true;
            idnatuoper.removeValidation('required');
            idnatuoper.value = null;
        }
    }

    // Valida o tipo de cálculo da data de saída e habilita o campo do parametro de cálculo.
    public alterarVisibilidadefieldCalculoDtSaida({ component } : any) {
        const nrcalculodtsaidanf = Metadata.getInstance<Select>('parametroFaturamentoTab-NRCALCULODTSAIDANF');

        if (!component.value || component.value === 1) {
            nrcalculodtsaidanf.readonly = true;
            nrcalculodtsaidanf.label = 'Dia fixo / Quantidade de dias para data de saída';
            nrcalculodtsaidanf.removeValidation('required');
            nrcalculodtsaidanf.value = null;
        } else {
            const nmlabel = ([2, 3].indexOf(component.value) !== -1) ? 'Dia fixo para data de saída' : 'Quantidade de dias para data de saída';
            nrcalculodtsaidanf.readonly = false;
            nrcalculodtsaidanf.label = nmlabel;
            nrcalculodtsaidanf.addValidation('required', {});
        }
    }

    // Valida se o valor digitado se aplica ao tipo de calculo escolhido.
    public validarParametrosCalculoDtSaida({ component } : any) {
        const idtpcalcdtsaidanf = Metadata.getInstance<Select>('parametroFaturamentoTab-IDTPCALCDTSAIDANF');

        if ([2, 3].indexOf(idtpcalcdtsaidanf.value) !== -1) {
            if (component.value > 31) {
                Utilities.alertMessage('Operação bloqueada!<br><br>Não é possível atribuir um dia fixo maior que 31 para o cálculo da data de saída da nota fiscal.', 'error');
                component.value = 0;
            }
        }
    }
}
