import { TekGrid } from '@zeedhi/teknisa-components-common';
import { Metadata, Http } from '@zeedhi/core';
import {
    LoadingService, DialogService,
} from '@zeedhi/common';
import { Utilities } from '@/util/Utilities';

export class Sm00050002QuantidadeMaximaParcelasPerfilAcessoController {
    public addPerfilAcesso() {
        const grid = Metadata.getInstance<TekGrid>('basic-tek-grid-perfil-acesso-quantidade-maxima-parcelas');
        const newId = 0;
        grid.addNewRow({ NRQTPARCMAXPERFACESS: newId }, 'start');
    }

    public afterModify() {
        Metadata.getInstance<TekGrid>('basic-tek-grid-perfil-acesso-quantidade-maxima-parcelas').datasource.get();
    }

    public insertQtdMaximaParcelasPerfilAcesso({ event }: any) {
        event!.preventDefault();
        const grid = Metadata.getInstance<TekGrid>('basic-tek-grid-perfil-acesso-quantidade-maxima-parcelas');
        const editedrows = grid.getEditedRows();

        let gridInvalido = true;
        if (editedrows.length > 0) {
            gridInvalido = editedrows.some((row) => row.NRPERFILACESSO == null || row.QTPARCMAXPERFACESS == null);
        }

        if (!gridInvalido) {
            LoadingService.show();
            Http.post('/saveQtdMaximaParcelasPerfilAcesso', editedrows)
                .then(() => {
                    grid.cancelEditedRows();
                    grid.reload();
                    Utilities.showSuccessMessage();
                    grid.datasource.get();
                }).catch((error) => {
                    if (error.response.data.error === 'EXISTE') {
                        Utilities.showDialogMessage('error', 'Operação bloqueada!', 'O Perfil de Acesso informado já está cadastrado.');
                    } else {
                        Utilities.showErrorMessage(error);
                    }
                }).finally(() => {
                    LoadingService.hide();
                });
        } else {
            Utilities.showAlertCamposObrigatorios();
        }
    }

    public showConfirmDeleteMessagePerfilAcesso({ event }: any) {
        event!.preventDefault();
        DialogService.show({
            name: 'buttons-dialog-perfil-acesso',
            type: 'warning',
            title: 'Excluir Registro',
            text: 'Deseja realmente excluir todos os registros selecionados?',
            buttons: [
                {
                    name: 'NAO',
                    component: 'ZdButton',
                    label: 'Não',
                    outline: true,
                    events: {
                        click: () => {
                            DialogService.hide();
                        },
                    },
                },
                {
                    name: 'SIM',
                    component: 'ZdButton',
                    label: 'Sim',
                    events: {
                        click: () => {
                            this.deleteQtdMaximaParcelasPerfilAcesso();
                            DialogService.hide();
                        },
                    },
                },
            ],
        });
    }

    public deleteQtdMaximaParcelasPerfilAcesso() {
        const grid = Metadata.getInstance<TekGrid>('basic-tek-grid-perfil-acesso-quantidade-maxima-parcelas');
        LoadingService.show();
        Http.post('/deleteQtdMaximaParcelasPerfilAcesso', grid.selectedRows)
            .then(() => {
                grid.selectedRows = [];
                grid.reload();
                Utilities.showSuccessMessage();
            }).catch((error) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                LoadingService.hide();
            });
    }
}
